# Generated by Django 2.2.11 on 2020-05-21 15:34

import uuid

import django.db.models.deletion
import enumfields.fields
from django.db import migrations, models

import arkindex.dataimport.models


def migrate_git_refs(apps, schema_editor):
    """
    Used to convert values on the ref attribute from Revision to a GitRef object.
    The ref attribute on Revision only supported branch-type references, so no need to
    worry about the type of the reference.

    Some references are duplicated on multiple revisions (e.g., two master branches), in
    which case the GitRef object is linked to the latest version returned by the default
    DB order. These values will be updated later when the GitLab webhook will be triggered.
    """
    db_alias = schema_editor.connection.alias
    Revision = apps.get_model('dataimport', 'Revision')
    for rev in Revision.objects.using(db_alias).filter(ref__isnull=False):
        ref = rev.repo.refs.filter(name=rev.ref).first()
        if ref:
            ref.revision = rev
            ref.save()
        else:
            rev.refs.create(
                name=rev.ref,
                type=arkindex.dataimport.models.GitRefType('branch'),
                repository=rev.repo
            )


class Migration(migrations.Migration):

    dependencies = [
        ('dataimport', '0007_worker_workerversion'),
    ]

    operations = [
        migrations.CreateModel(
            name='GitRef',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('type', enumfields.fields.EnumField(enum=arkindex.dataimport.models.GitRefType, max_length=10)),
                ('name', models.CharField(max_length=250)),
                ('repository', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='refs',
                    to='dataimport.Repository'
                )),
                ('revision', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='refs',
                    to='dataimport.Revision'
                )),
            ],
            options={
                'unique_together': {('name', 'repository')},
            },
        ),
        migrations.RunPython(
            migrate_git_refs,
            reverse_code=migrations.RunPython.noop,
        ),
        migrations.AddField(
            model_name='repository',
            name='git_ref_revisions',
            field=models.ManyToManyField(through='dataimport.GitRef', to='dataimport.Revision'),
        ),
        migrations.RemoveField(
            model_name='revision',
            name='ref',
        ),
    ]
