from django.conf import settings
from rest_framework.exceptions import ValidationError
from rest_framework.generics import CreateAPIView

from arkindex.documents.serializers.admin import ReindexConfigSerializer
from arkindex.project.permissions import IsAdminUser


class ReindexStart(CreateAPIView):
    """
    Run an ElasticSearch indexation from the API.
    """
    permission_classes = (IsAdminUser, )
    serializer_class = ReindexConfigSerializer
    openapi_overrides = {
        'operationId': 'Reindex',
        'description': 'Manually reindex elements, transcriptions and entities for search APIs',
        'tags': ['management']
    }

    def post(self, *args, **kwargs):
        if not settings.ARKINDEX_FEATURES['search']:
            raise ValidationError(['Reindexation is unavailable due to the search feature being disabled.'])
        return super().post(*args, **kwargs)
