#!/usr/bin/env python3
import logging
import uuid

from django.core.management.base import BaseCommand

from arkindex.documents.models import Element

logging.basicConfig(
    level=logging.INFO,
    format='[%(levelname)s] %(message)s',
)
logger = logging.getLogger(__name__)


def valid_uuid(value):
    try:
        return uuid.UUID(value)
    except ValueError:
        raise Exception(f"Not an UUID: {value}")


class Command(BaseCommand):
    help = 'Delete an element and its children'

    def add_arguments(self, parser):
        super().add_arguments(parser)
        parser.add_argument(
            'element_id',
            help='ID of the top element to delete (multiple supported)',
            type=valid_uuid,
            nargs='+',
        )

    def handle(self, element_id, verbosity=0, **options):
        for element in Element.objects.filter(id__in=element_id):
            logger.info(f"Deleting {element.id} : {element}")
            element.delete()
