# Generated by Django 3.1 on 2020-09-01 07:48

from django.db import migrations

FORWARD_SQL = [
    # Use a temporary table here to iterate over transcriptions just once before deleting,
    # causing this migration to only take a few minutes
    # Note the strange join conditions as either source_id or worker_version_id are NULL,
    # which causes a NATURAL JOIN or a JOIN … USING to fail since comparing NULLs returns NULL.
    """
    CREATE TEMPORARY TABLE duplicate_ids AS
    WITH filters AS (
        SELECT
            sub.*,
            FIRST_VALUE(id) OVER (
                PARTITION BY
                    transcription.element_id,
                    transcription.source_id,
                    transcription.worker_version_id
            ) AS keep_id
        FROM documents_transcription transcription
        INNER JOIN (
            SELECT element_id, source_id, worker_version_id
            FROM documents_transcription
            GROUP BY element_id, source_id, worker_version_id
            HAVING COUNT(*) > 1
        ) sub ON (
            sub.element_id = transcription.element_id AND (
                sub.source_id = transcription.source_id
                OR sub.worker_version_id = transcription.worker_version_id
            )
        )
    )
    SELECT id
    FROM documents_transcription transcription
    INNER JOIN filters ON (
        filters.element_id = transcription.element_id AND (
            filters.source_id = transcription.source_id
            OR filters.worker_version_id = transcription.worker_version_id
        )
    )
    WHERE keep_id != id;
    """,
    # Remove any TranscriptionEntity that could be linked to the duplicate transcriptions
    """
    DELETE FROM documents_transcriptionentity transcriptionentity
    USING duplicate_ids
    WHERE transcriptionentity.transcription_id = duplicate_ids.id;
    """,
    # Remove duplicate transcriptions
    """
    DELETE FROM documents_transcription transcription
    USING duplicate_ids
    WHERE transcription.id = duplicate_ids.id;
    """,
    'DROP TABLE duplicate_ids;',
]


class Migration(migrations.Migration):

    dependencies = [
        ('documents', '0021_move_transcriptions'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='transcription',
            name='zone',
        ),
        # Remove the few remaining transcriptions that would break the unique constraints we will add in documents.0023.
        # Those are transcriptions from the same source, on the same element, with the exact same zones.
        # This query is rather complex as we want to only remove duplicates, and window functions have their limits,
        # but the GROUP BY…HAVING will quickly exclude most of the table so it isn't slow.
        migrations.RunSQL(
            FORWARD_SQL,
            reverse_sql=migrations.RunSQL.noop,
            elidable=True,
        ),
    ]
