from django.core.exceptions import ValidationError
from django.core.management.base import CommandError
from django.db.models import Model

from arkindex.dataimport.models import DataImport, Repository
from arkindex.documents.models import Corpus, Element


class ModelArgument(object):
    model = None
    text_search_field = 'name'

    def __init__(self, many=False, **filters):
        assert issubclass(self.model, Model)
        self.many = many
        self.filters = filters

    def __call__(self, arg):
        qs = self.model.objects.filter(**self.filters)
        try:
            return qs.get(pk=arg)
        except (ValidationError, Element.DoesNotExist):
            pass

        try:
            return self.text_search(qs, arg)
        except self.model.DoesNotExist:
            raise CommandError('{} "{}" does not exist'.format(self.model.__name__, arg))
        except self.model.MultipleObjectsReturned:
            raise CommandError('"{}" matches multiple {} instances'.format(arg, self.model.__name__))

    def text_search(self, qs, arg):
        if not self.text_search_field:
            raise self.model.DoesNotExist
        text_filter = {'{}__icontains'.format(self.text_search_field): arg}
        if self.many:
            return qs.filter(**text_filter)
        else:
            return qs.get(**text_filter)


class CorpusArgument(ModelArgument):
    model = Corpus


class ElementArgument(ModelArgument):
    model = Element


class DataImportArgument(ModelArgument):
    model = DataImport
    text_search_field = None


class RepositoryArgument(ModelArgument):
    model = Repository
    text_search_field = 'url'
