from unittest import TestCase
from unittest.mock import patch

from django.test import override_settings

from arkindex.project.openapi import SchemaGenerator


class TestSchemaGenerator(TestCase):

    @override_settings(VERSION='0.0')
    @patch('arkindex.project.openapi.generators.yaml.safe_load', return_value={})
    def test_auto_version(self, mock):
        """
        Ensure the schema version is set to the auto-detected backend's version
        """
        schema = SchemaGenerator().get_schema()
        self.assertEqual(schema['info']['version'], '0.0')

    def test_patch_paths_assertions(self):
        gen = SchemaGenerator()

        with self.assertRaisesRegex(AssertionError, 'OpenAPI path /somewhere/ does not exist'):
            gen.patch_paths(
                paths={},
                patches={'/somewhere/': {}}
            )

        with self.assertRaisesRegex(AssertionError, 'Method patch on OpenAPI path /somewhere/ does not exist'):
            gen.patch_paths(
                paths={
                    '/somewhere/': {
                        'get': {}
                    }
                },
                patches={
                    '/somewhere/': {
                        'patch': {}
                    }
                }
            )

    def test_patch_paths(self):
        patches = {
            '/somewhere/': {
                'get': {
                    'operationId': 'PatchedSomewhere'
                },
                'post': {
                    'responses': {
                        '409': {
                            'description': 'Conflict'
                        }
                    }
                }
            }
        }
        expected = {
            '/somewhere/': {
                'get': {
                    'operationId': 'PatchedSomewhere'
                },
                'post': {
                    'operationId': 'PostSomewhere',
                    'responses': {
                        '418': {
                            'description': "I'm a teapot"
                        },
                        '409': {
                            'description': 'Conflict'
                        }
                    }
                }
            }
        }
        actual = {
            '/somewhere/': {
                'get': {
                    'operationId': 'GetSomewhere',
                },
                'post': {
                    'operationId': 'PostSomewhere',
                    'responses': {
                        '418': {
                            'description': "I'm a teapot",
                        }
                    }
                }
            }
        }
        SchemaGenerator().patch_paths(actual, patches)

        self.assertDictEqual(actual, expected)
